"""
eikon_template.py: template for fetching data from Eikon Python API.

The current version of Eikon API has two limitations which this template accounts for:
1. Size of data fields per call is limited. Thus fetching data in blocks is necessary
2. The request can sometimes fail due to server errors. Thus recursion is used to send the request again.
"""

import tkinter as tk
from tkinter import filedialog

import eikon as ek
import pandas as pd


def get_path():
    """Asks user for path of input file."""
    root = tk.Tk()
    root.withdraw()
    input_file = filedialog.askopenfilename(title="Select file with raw data",
                                            filetypes = (("CSV files","*.csv"),))
    output_file = input_file.rsplit('.', 1)[0] + '_fetched'

    return input_file, output_file

def load_file(input_df):
    """Loads raw data .csv file and extracts list of CUSIPs, ISINs or Tickers.

    params:
        input_df: String
    """
    df = pd.read_csv(input_df)
    # Replace 'identifier' with column name where CUSIP, ISIN or Ticker is stored
    assets = list(df["identifier"].drop_duplicates())

    return assets

def get_data(input_df, output_name, asset_list):
    """Fetches data for asset list of input_df from eikon and stores in .csv file.
    
    params:
        input_df: String
        output_name: String
        asset_list: list
    """
    assets = asset_list

    # Choose fields for which data should be fetched. 
    # The true name for the fields can be found using 'Item Data Browser' in Eikon.
    eikon_fields = ["TR.CommonName", "TR.Volume"]

    # Prepare header for summary df
    #columns_df = ["Instrument"] + eikon_fields
    columns_df = [ "Id"] + eikon_fields

    agg_df = pd.DataFrame(columns=columns_df)
    print("Fetch data for file:", input_df, "-- Total number of assets:", len(assets))

    # Fetch data in chunks of size 2000 and aggregate in pandas dataframe
    chunk_size = 2000
    for i in range(0, len(assets), chunk_size):
        assets_subset = assets[i:i + chunk_size]
        tmp_df = fetch_data(assets_subset, eikon_fields, 1)
        tmp_df.columns = columns_df
        print("Shape of requested data: ", tmp_df.shape)
        agg_df = agg_df.append(tmp_df)
        print("Shape of aggregated dataframe:", agg_df.shape)

    print("Write aggregated data to .csv file")
    agg_df.to_csv(output_name + ".csv", encoding="utf-8")
    print("Wrote aggregated data to ", output_name + ".csv")

def fetch_data(asset_list, fields, attempt):
    """Fetches data using Eikon API.
    
    Employs recursive calls of get_data() method of Eikon, as request can 
    fail sometimes.
    
    params:
        asset_list: list
        fields: list
        attempt: int
    """
    attempt_counter = attempt
    if attempt_counter > 10:
        print("Exceeded maximum number of attempts. Try smaller chunks")
        return    
    try:
        df, err = ek.get_data(asset_list, fields)
    except Exception as e:
        attempt_counter += 1
        print(e)
        print("Failed to fetch data. Retry again in attempt", attempt_counter)
        return fetch_data(asset_list, fields, attempt_counter)
    else:
        return df

def main():
    """Executes program."""
    ek.set_app_key('1c061ffd70ee4ad2b369888440796a4e2f550ddf')

    input_file, output_file = get_path()
    asset_list = load_file(input_file)
    get_data(input_file, output_file, asset_list)

if __name__ == "__main__":
    main()